import java.util.*;
import javax.management.*;
import javax.management.remote.*;
import javax.naming.Context;

// Per JMX MBean-Attribute auslesen und MBean-Methoden aufrufen
public class JmxMBeanLesenAufrufen
{
   @SuppressWarnings("fallthrough")
   public static void main( String[] args ) throws Exception
   {
      String attributeName = "Uptime";
      String objectName    = "java.lang:type=Runtime";
      String url           = "localhost:8686";
      String usr           = null;
      String pwd           = null;
      String opt           = null;
      switch( Math.min( args.length, 6 ) ) {
         case 6: opt           = args[5];
         case 5: pwd           = args[4];
         case 4: usr           = args[3];
         case 3: url           = args[2];
         case 2: objectName    = args[1];
         case 1: attributeName = args[0];
      }
      JMXConnector jmxConnector = null;
      try {
         jmxConnector = getJMXConnector( url, usr, pwd );
         MBeanServerConnection mBeanServerConn = jmxConnector.getMBeanServerConnection();
         if( opt != null && opt.equals("all") ) showAllObjectNames( mBeanServerConn );
         createJRockitConsoleMBean( objectName, mBeanServerConn );
         Set<ObjectName> objectNames = mBeanServerConn.queryNames( new ObjectName( objectName ), null );
         for( ObjectName on : objectNames ) {
            if( attributeName.equals("invoke") && args.length > 5 ) {
               String methodName = args[5];
               String[] parms = null;
               if( args.length > 6 ) {
                  parms = new String[args.length - 6];
                  System.arraycopy( args, 6, parms, 0, parms.length );
               }
               System.out.println( "" + invoke( methodName, parms, on, mBeanServerConn ) +
                                   " = invoke " + methodName + " " + Arrays.toString( parms ) );
            } else {
               Object attr = mBeanServerConn.getAttribute( on, attributeName );
               System.out.println( "" + attr + " = " + attributeName + " (" + on + ")" );
            }
         }
      } finally {
         if( jmxConnector != null ) jmxConnector.close();
      }
   }

   // Rufe MBean-Methode auf
   // parms: paarweise Klasse und Parameterwert, z.B.:
   //        [Integer, 12, Integer, 34]
   //        oder
   //        [int, 12, int, 34]
   // Die Klassen (bzw. Wrapper-Klassen) muessen einen Konstruktor mit String-Parameter haben
   static Object invoke( String methodName, String[] parms, ObjectName on, MBeanServerConnection mBeanServerConn ) throws Exception
   {
      if( methodName == null || on == null || mBeanServerConn == null ) return null;
      Object[] oa = null;
      String[] sa = null;
      if( parms != null && parms.length >= 2 ) {
         final Map<String,Class<?>[]> PRIMITIVE_TYPEN = new HashMap<String,Class<?>[]>();
         PRIMITIVE_TYPEN.put( "boolean", new Class<?>[] { boolean.class, Boolean.class } );
         PRIMITIVE_TYPEN.put( "int",     new Class<?>[] { int.class,     Integer.class } );
         PRIMITIVE_TYPEN.put( "long",    new Class<?>[] { long.class,    Long.class    } );
         PRIMITIVE_TYPEN.put( "double",  new Class<?>[] { double.class,  Double.class  } );
         oa = new Object[parms.length / 2];
         sa = new String[parms.length / 2];
         for( int i = 0; i < parms.length - 1; i++ ) {
            // Sind Parameter primitive Typen?
            Class<?>[] classForSigAndObj = PRIMITIVE_TYPEN.get( parms[i] );
            // Klassen als Parameter-Typen:
            if( classForSigAndObj == null ) {
               classForSigAndObj = new Class<?>[2];
               try {
                  classForSigAndObj[0] = Class.forName( "java.lang." + parms[i] );
               } catch( ClassNotFoundException ex ) {
                  classForSigAndObj[0] = Class.forName( parms[i] );
               }
               classForSigAndObj[1] = classForSigAndObj[0];
            }
            oa[i/2] = classForSigAndObj[1].getConstructor( String.class ).newInstance( parms[++i] );
            sa[i/2] = classForSigAndObj[0].getName();
         }
      }
      return mBeanServerConn.invoke( on, methodName, oa, sa );
   }

   // Zeige alle vorhandenen ObjectNames
   static void showAllObjectNames( MBeanServerConnection mBeanServerConn ) throws Exception
   {
      System.out.println( "\n---- showAllObjectNames ----" );
      Iterator<?> itr = mBeanServerConn.queryNames( null, null ).iterator();
      while( itr.hasNext() ) System.out.println( itr.next() );
      System.out.println( "----------------------------\n" );
   }

   // Vor einer Abfrage von JRockit-Attributen muss die JRockitConsole-MBean instanziiert werden
   static void createJRockitConsoleMBean( String objectName, MBeanServerConnection mBeanServerConn ) throws Exception
   {
      if( objectName == null || !objectName.startsWith( "oracle.jrockit.management" ) ) return;
      try {
         mBeanServerConn.getMBeanInfo( new ObjectName( "oracle.jrockit.management:type=JRockitConsole" ) );
      } catch( InstanceNotFoundException ex ) {
         mBeanServerConn.createMBean( "oracle.jrockit.management.JRockitConsole", null );
      }
   }

   // JMX-Connection
   static JMXConnector getJMXConnector( String url, String usr, String pwd ) throws Exception
   {
      String serviceUrl = "service:jmx:rmi:///jndi/rmi://" + url + "/jmxrmi";
      if( usr == null || usr.trim().length() <= 0 || pwd == null || pwd.trim().length() <= 0 ) {
         return JMXConnectorFactory.connect( new JMXServiceURL( serviceUrl ) );
      }
      Map<String,Object> envMap = new HashMap<String,Object>();
      envMap.put( "jmx.remote.credentials", new String[] { usr, pwd } );
      envMap.put( Context.SECURITY_PRINCIPAL, usr );
      envMap.put( Context.SECURITY_CREDENTIALS, pwd );
      return JMXConnectorFactory.connect( new JMXServiceURL( serviceUrl ), envMap );
   }
}
