package de.meinefirma.meinprojekt;

import java.lang.management.*;
import java.util.List;
import javax.management.MBeanServerConnection;
import org.junit.*;

public class GcCpuTxMBeanServletTest
{
   @Test
   public void testCalculateCpuTimePercent()
   {
      Assert.assertEquals(   0, GcCpuTxMBeanUtil.calculateCpuTimePercent(          0,        0,   1,  0, 1 ) );
      Assert.assertEquals(   0, GcCpuTxMBeanUtil.calculateCpuTimePercent(   30922111, 30222111, 100, 10, 2 ) );
      Assert.assertEquals(  11, GcCpuTxMBeanUtil.calculateCpuTimePercent(   50222111, 30222111, 100, 10, 2 ) );
      Assert.assertEquals( 100, GcCpuTxMBeanUtil.calculateCpuTimePercent(  189322111, 10222111, 100, 10, 2 ) );
      Assert.assertEquals( 100, GcCpuTxMBeanUtil.calculateCpuTimePercent( 1900222111, 10222111, 100, 10, 2 ) );
   }

   @Test
   public void testCalculateGarbageCollectionTimePercent()
   {
      Assert.assertEquals(   0., GcCpuTxMBeanUtil.calculateGarbageCollectionTimePercent(    0,  0,   1,  0, 1 ), 0.1 );
      Assert.assertEquals(   0., GcCpuTxMBeanUtil.calculateGarbageCollectionTimePercent(   30, 30, 100, 10, 2 ), 0.1 );
      Assert.assertEquals(  10., GcCpuTxMBeanUtil.calculateGarbageCollectionTimePercent(   48, 30, 100, 10, 2 ), 0.1 );
      Assert.assertEquals( 100., GcCpuTxMBeanUtil.calculateGarbageCollectionTimePercent(  190, 10, 100, 10, 2 ), 0.1 );
      Assert.assertEquals( 100., GcCpuTxMBeanUtil.calculateGarbageCollectionTimePercent( 1900, 10, 100, 10, 2 ), 0.1 );
   }

   @Test
   public void testCalculatePerSecond()
   {
      long[]   counts     = new long[] { 42, 2 };
      long[]   lastCounts = new long[] {  7, 1 };
      String[] result     = GcCpuTxMBeanUtil.calculatePerSecond( counts, lastCounts, 210, 10 );
      Assert.assertEquals( "175", result[0] );
      Assert.assertEquals( "5,0", result[1] );
   }

   @Test
   public void testReadGarbageCollectionTimeMs()
   {
      MBeanServerConnection        mbs          = ManagementFactory.getPlatformMBeanServer();
      RuntimeMXBean                rtMXBean     = ManagementFactory.getRuntimeMXBean();
      OperatingSystemMXBean        opSystMXBean = ManagementFactory.getOperatingSystemMXBean();
      List<GarbageCollectorMXBean> gcMXBeans    = GcCpuTxMBeanUtil.getGarbageCollectorMXBeans( mbs );

      Assert.assertNotNull( "mbs", mbs );
      Assert.assertNotNull( "rtMXBean", rtMXBean );
      Assert.assertNotNull( "opSystMXBean", opSystMXBean );
      Assert.assertNotNull( "gcMXBeans", gcMXBeans );
      Assert.assertTrue( "cpuCount", opSystMXBean.getAvailableProcessors() >= 1 );
      Assert.assertTrue( "rtMXBean.getUptime()", rtMXBean.getUptime() > 0 );
      Assert.assertTrue( "readCpuTimeNs", GcCpuTxMBeanUtil.readCpuTimeNs( mbs ) > 0 );
      Assert.assertTrue( "readGarbageCollectionTimeMs", GcCpuTxMBeanUtil.readGarbageCollectionTimeMs( gcMXBeans ) >= 0 );
   }

   @Test
   public void testReadJRockitWebLogic()
   {
      MBeanServerConnection mbs = ManagementFactory.getPlatformMBeanServer();
      Assert.assertNotNull( GcCpuTxMBeanUtil.readCpuLoadsJRockit( mbs ) );
      Assert.assertNotNull( GcCpuTxMBeanUtil.readTxCountsWebLogic( mbs ) );
   }

   @Test
   public void testGcCpuTxMBeanTimerTask()
   {
      GcCpuTxMBeanTimerTask mbtt = new GcCpuTxMBeanTimerTask( null, "target/CpuGcProzentLog.csv", true );
      Assert.assertNull( mbtt.getResult() );
      mbtt.run();
      try { Thread.sleep( 550 ); } catch( InterruptedException e ) {/*ok*/}
      mbtt.run();
      String s = mbtt.getResult();
      Assert.assertTrue( "mbtt.getResult", s != null && s.length() >= 24 );
   }
}
