import java.util.*;
import javax.management.*;
import javax.management.remote.*;
import javax.naming.Context;

public class Weblogic10ServerRuntimeMBean
{
   public static void main( String[] args ) throws Exception
   {
      String url = "t3://localhost:7001";
      String usr = "weblogic";
      String pwd = "weblogic0";
      JMXConnector jmxConnector = null;
      try {
         jmxConnector = getJMXConnector( url, usr, pwd );
         MBeanServerConnection mBeanServerConn = jmxConnector.getMBeanServerConnection();
         if( args != null && args.length > 0 && args[0].equals("all") ) {
            showAllObjectNames( mBeanServerConn );
         }
         showServerRuntimesJtaJms( mBeanServerConn );
      } finally {
         if( jmxConnector != null ) jmxConnector.close();
      }
   }

   static void showAllObjectNames( MBeanServerConnection con ) throws Exception
   {
      System.out.println( "\n---- showAllObjectNames ----" );
      Iterator<?> itr = con.queryNames( null, null ).iterator();
      while( itr.hasNext() ) System.out.println( itr.next() );
   }

   static void showServerRuntimesJtaJms( MBeanServerConnection con ) throws Exception
   {
      System.out.println( "\n---- showServerRuntimesJtaJms ----" );
      ObjectName service = new ObjectName(
            "com.bea:Name=DomainRuntimeService," +
            "Type=weblogic.management.mbeanservers.domainruntime.DomainRuntimeServiceMBean" );
      ObjectName[] srv = (ObjectName[]) con.getAttribute( service, "ServerRuntimes" );
      for( int i = 0; i < srv.length; i++ ) {
         System.out.println("ServerRuntime: " + getDescription(srv[i], con));
         System.out.println("Name: " + con.getAttribute(srv[i], "Name"));
         System.out.println("  State:                            " + con.getAttribute(srv[i], "State"));
         System.out.println("  ListenAddress:                    " + con.getAttribute(srv[i], "ListenAddress"));
         System.out.println("  ListenPort:                       " + con.getAttribute(srv[i], "ListenPort"));
         System.out.println("  DefaultURL:                       " + con.getAttribute(srv[i], "DefaultURL"));
         System.out.println("  ActivationTime:                   " + con.getAttribute(srv[i], "ActivationTime"));
         System.out.println("  HealthState:                      " + con.getAttribute(srv[i], "HealthState"));
         ObjectName jta = (ObjectName) con.getAttribute(srv[i], "JTARuntime");
         System.out.println("JTARuntime: " + getDescription(jta, con));
         System.out.println("  JTA-ActiveTransactionsTotalCount: " + con.getAttribute(jta, "ActiveTransactionsTotalCount"));
         System.out.println("  JTA-TransactionTotalCount:        " + con.getAttribute(jta, "TransactionTotalCount"));
         System.out.println("  JTA-HealthState:                  " + con.getAttribute(jta, "HealthState"));
         ObjectName jms = (ObjectName) con.getAttribute(srv[i], "JMSRuntime");
         System.out.println("JMSRuntime: " + getDescription(jms, con));
         System.out.println("  JMS-ConnectionsTotalCount:        " + con.getAttribute(jms, "ConnectionsTotalCount"));
         System.out.println("  JMS-JMSServersTotalCount:         " + con.getAttribute(jms, "JMSServersTotalCount"));
         System.out.println("  JMS-HealthState:                  " + con.getAttribute(jms, "HealthState"));
      }
   }

   static JMXConnector getJMXConnector( String url, String usr, String pwd ) throws Exception
   {
      String serviceUrl = "service:jmx:" + url + "/jndi/weblogic.management.mbeanservers.domainruntime";
      Map<String, Object> envMap = new HashMap<String, Object>();
      envMap.put( Context.SECURITY_PRINCIPAL, usr );
      envMap.put( Context.SECURITY_CREDENTIALS, pwd );
      envMap.put( JMXConnectorFactory.PROTOCOL_PROVIDER_PACKAGES, "weblogic.management.remote" );
      return JMXConnectorFactory.connect( new JMXServiceURL( serviceUrl ), envMap );
   }

   static String getDescription( ObjectName on, MBeanServerConnection con ) throws Exception
   {
      String s = con.getMBeanInfo( on ).getDescription();
      int n = s.indexOf( '<' );
      return (n > 10) ? s = s.substring( 0, n ) : s;
   }
}
